
{ NAME: antinag.pas                                                         }
{ VERSION: 2.0                                                              }
{ DATE: October 1994.                                                       }
{ DESCRIPTION: To decode NAGRAVISON images.                                 }
{ ENVIRONMENT: Turbo Pascal 7.0                                             }

program AntiNagravision;

uses crt,dos;

const
  max_res=512;      { Maximun resolution for PGM file }

type
  tbuffer=array [1..max_res] of byte;
  tlines=array [1..max_res] of word;
  pgm_header=record
    x_max,y_max,gray_scale,len:word;
  end;

var
  buffer1,buffer2:tbuffer;
  new_number_lines:tlines;
  head:pgm_header;
  f_src,f_dest:file;
  head_ok:boolean;

{--------------------------------------------------------------------------}
procedure process_header(var f_src,f_dest:file;var head:pgm_header;
                          var buffer:tbuffer;var head_ok:boolean);
var
  w,code:word;
  str_aux:string;

procedure get_text(var buffer1:tbuffer;var i:word;var out:string);
begin
  out:='';
  inc(i);
  while buffer1[i]<>$0a do
    begin
      out:=out+chr(buffer1[i]);
      inc(i);
    end;
end;

begin
  write('- Processing header... ');
  blockread(f_src,buffer,max_res,code);
  head.len:=0;
  get_text(buffer,head.len,str_aux);
  head_ok:=(str_aux='P5');
  if head_ok
    then
      begin
        get_text(buffer,head.len,str_aux);
        w:=pos(' ',str_aux);
        val(copy(str_aux,1,w-1),head.x_max,code);
        val(copy(str_aux,w+1,ord(str_aux[0])-w),head.y_max,code);
        get_text(buffer,head.len,str_aux);
        val(str_aux,head.gray_scale,w);
        blockwrite(f_dest,buffer,head.len);
        if (head.x_max>max_res) or (head.y_max>max_res)
          then
            head_ok:=false;
      end;
  if head_ok
    then
      writeln('OK')
    else
      writeln('-> Incorrect PGM header. <-');
end;
{--------------------------------------------------------------------------}
procedure get_line(var f_src:file;var buffer:tbuffer;num_line:word;
                   var head:pgm_header);
var
  file_pos:longint;
begin
  file_pos:=longint(head.len)+(longint(head.x_max)*(longint(num_line)-1));
  seek(f_src,file_pos);
  blockread(f_src,buffer,head.x_max);
end;
{--------------------------------------------------------------------------}
procedure resort_lines(var f_src:file;var head:pgm_header;
                       var new_number_lines:tlines;var buffer1:tbuffer);
var
  current_line,probe_line,count,i,min_error,error,line_min_error,
  hour,minute,second,sec100:word;
  elapsed_time:real;
  x,y:byte;
begin
  gettime(hour,minute,second,sec100);
  write('- Resorting lines... ');
  x:=wherex;
  y:=wherey;
  fillchar(new_number_lines,sizeof(new_number_lines),0);
  new_number_lines[1]:=1;
  count:=1;
  current_line:=1;
  repeat
    inc(count);
    gotoxy(x,y);
    write(head.y_max-count:3);
    get_line(f_src,buffer1,current_line,head);
    min_error:=$ffff;
    for probe_line:=1 to head.y_max do
      begin
        if (new_number_lines[probe_line]=0)
          then
            begin
              get_line(f_src,buffer2,probe_line,head);
              error:=0;
              for i:=1 to head.x_max do error:=error+abs(buffer2[i]-buffer1[i]);
              if (error<min_error)
                then
                  begin
                    min_error:=error;
                    line_min_error:=probe_line;
                  end;
            end;
      end;
    new_number_lines[line_min_error]:=count;
    current_line:=line_min_error;
  until count=(head.y_max);
  elapsed_time:=(hour*3600+minute*60+second);
  gettime(hour,minute,second,sec100);
  elapsed_time:=(hour*3600+minute*60+second)-elapsed_time;
  gotoxy(x,y);
  writeln('OK (',elapsed_time/60:0:2,' minutes).');
end;
{--------------------------------------------------------------------------}
procedure save_lines(var f_src,f_dest:file;var head:pgm_header;
                     var new_number_lines:tlines;var buffer:tbuffer);
var
  i:word;
  file_pos:longint;
begin
  write('- Saving resorted lines... ');
  for i:=1 to head.y_max do
    begin
      get_line(f_src,buffer,i,head);
      file_pos:=longint(head.len)+
                (longint(head.x_max)*(longint(new_number_lines[i])-1));
      seek(f_dest,file_pos);
      blockwrite(f_dest,buffer,head.x_max);
    end;
  writeln('OK');
end;
{--------------------------------------------------------------------------}
{--------------------------------------------------------------------------}
begin
  writeln('ANTINAG.EXE v2.0 - October 1994 - By Toni Cebrian -');
  writeln('To decode NAGRAVISON images.');
  writeln;
  if (paramcount<>2)
    then
      begin
        writeln(' Use: ANTINAG file_src.pgm file_dest.pgm');
        writeln;
        writeln(' Example: ANTINAG nagra1.pgm nagra1-d.pgm');
      end
    else
      begin
        assign(f_src,paramstr(1));
        reset(f_src,1);
        assign(f_dest,paramstr(2));
        rewrite(f_dest,1);
        process_header(f_src,f_dest,head,buffer1,head_ok);
        if head_ok
          then
            begin
              resort_lines(f_src,head,new_number_lines,buffer1);
              save_lines(f_src,f_dest,head,new_number_lines,buffer1);
            end;
        close(f_src);
        close(f_dest);
      end;
end.
