#include <stdio.h>

/*** #defines for COMMS ***/

#define    COM1INT 0x01
#define    COM2INT 0x02
#define    TIMEINT 0x04

#define    COM1    0x3F8
#define    COM2    0x2F8

#define    DATA8   0x03
#define    DATA7   0x02
#define    STOP1   0x00
#define    STOP2   0x04
#define    PEVEN   0x18
#define    PODD    0x08
#define    PNONE   0x00


#define    BD_9600 12
#define    BD_4800 24
#define    BD_2400 48
#define    BD_1200 96
#define    BD_600  192
#define    BD_300  384



#define    ST_DCD  0x80
#define    ST_RI   0x40
#define    ST_DSR  0x20
#define    ST_CTS  0x10
#define    ST_BRK  0x08
#define    ST_FRM  0x04
#define    ST_PTY  0x02
#define    ST_OVR  0x01
#define    ST_ERR  0x0f


#define    D_NORM  0x00    /* Serial data is normal   */
#define    D_REV   0x01    /* Serial data is reversed */
#define    D_INV   0x02    /* Serial data is inverted */


/** Defines for VIDEOCRYPT packets **/

#define    D_CARD      0x00        /* Dirn to CARD */
#define    D_DECODER   0x01        /* Dirn to DECODER */

#define S_RESET     0
#define S_RELEASED  1
#define S_STIMER    2
#define S_SENDSTR   3
#define S_SENDECHO  4
#define S_WAITCMD   5
#define S_GETCMD    6
#define S_GETCMD2   7
#define S_GETLEN    8
#define S_CMDREPLY  9
#define S_SENDTERM  10
#define S_WAIT      11
#define S_DIRN      12


typedef    unsigned int    word;
typedef    unsigned char   byte;

typedef    struct  {
                   word    port;
                   word    format;
                   word    baud;
                   }
                   portset;

typedef    struct  {
                   word    cmdword;
                   byte    len;
                   byte    dirn;
                   byte *  dataptr;
                   }
                   cmddef;

extern byte    com1stat;
extern byte    com2stat;

int    showall = 1;

unsigned char rxdata[ 64 ];
unsigned char cardid[ 16 ] = { 0x3f,0xfa,0x11,0x25,0x05,0x00,0x01,0xb0,
                               0x02,0x3b,0x34,0x4d,0x59,0x00,0x81,0x80 };


unsigned char ser_num[ 6 ] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
unsigned char nul_msg[ 25] = { 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
                               0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                               0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                               0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
unsigned char cod_msg[ 16] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                               0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                               0x00, 0x00, 0x00, 0x00 };
unsigned char seq_msg[ 8 ] = { 0x00, 0x00, 0x00, 0x00,
                               0X00, 0X00, 0X00, 0X00 };

unsigned char terminator[ 2 ] = {0x90, 0x00};

cmddef  instructs[ 24 ] = {
                             { 0x5370, 0x06, D_DECODER, ser_num },
                             { 0x5374, 0x20, D_CARD, NULL },
                             { 0x537a, 0x19, D_DECODER, nul_msg },
                             { 0x537c, 0x10, D_DECODER, cod_msg },
                             { 0x5378, 0x08, D_DECODER, seq_msg },
                             {      0,    0, D_CARD, NULL }
                          };

int     state     = S_WAIT;

void set_state( byte newstate );
cmddef * chk_cmd( word command );
extern int com2_wr( unsigned char data, unsigned char fn );
extern int timeout1;


main( int argc, char * argv[] )
{
   portset comport;
   int     rxval,inst,rlen,rxlen,txlen,i;
   char    fn        = D_REV + D_INV;
   byte    dcd_stat,waitbyte;
   FILE  * outfile   = NULL;
   int     next_state;
   byte * rxptr;
   byte * txptr;
   unsigned int cmdword;
   cmddef * chkptr;


   if ( argc > 1 )
   {
       outfile = fopen( argv[1], "wb" );
   }


   settimer();
   setint1b();
   setint1();
   setint2();

   comport.port    = COM2;
   comport.format  = DATA8 + STOP1 + PODD;
   comport.baud    = BD_9600;

   setcom( &comport );

   set_state(S_WAIT); 
   dcd_stat = com2stat;
   waitbyte = 0;

   timer1(1);

   while (!(keypress()))
   {
       if ( (com2stat & ST_DCD) != dcd_stat )  /** DCD changed    **/
       {
           dcd_stat = com2stat & ST_DCD;
           if ( dcd_stat )                     /** Reset Asserted **/
           {
               waitbyte = 0;
               set_state(S_RESET);
           }
           else
               set_state(S_RELEASED);             /** Reset Released **/
       }

       if ( waitbyte )
       {
          if (( rxval = com2rd(fn)) != -1)
          {
             rxval &= 0xff;
             printf("%2.2X ",rxval);
             waitbyte--;
          }
       }
       else switch (state)
       {
       case    S_RESET     : break;
       case    S_RELEASED  : timer1( 1 );  /* Wait 2 x 54.92 ms */
                             set_state(S_STIMER);
                             break;
       case    S_STIMER    : if (timeout1)
                             {
                                next_state = S_WAITCMD;
                                set_state(S_SENDSTR);
                                txlen      = 16;
                                txptr      = &cardid[0];
                             }
                             break;
       case    S_SENDSTR   : if (com2wr( fn, *txptr ) != -1)
                             {
                                txptr++;
                                waitbyte   = 1;
                                set_state( S_SENDECHO);
                             }
                             break;
       case    S_SENDECHO  : if ( --txlen )
                                set_state(S_SENDSTR);
                             else
                                set_state(next_state);
                             break;
       case    S_WAITCMD   : printf("\n");
                             waitbyte = 1;
                             set_state(S_GETCMD);
                             break;
       case    S_GETCMD    : if (startcmdwd( rxval & 0xff))
                             {
                                cmdword  = rxval << 8;
                                waitbyte = 1;
                                set_state(S_GETCMD2);
                             }
                             else set_state(S_WAITCMD);
                             break;
       case    S_GETCMD2   : cmdword += rxval;
                             waitbyte = 3;
                             set_state(S_GETLEN);
                             break;
       case    S_GETLEN    : rxlen    = rxval;
                             printf("Received Command\n");
                             set_state(S_CMDREPLY);
                             break;
       case    S_CMDREPLY  : if ( com2wr( fn, cmdword & 0xff ) != -1)
                             {
                               printf("\n");
                               waitbyte = 1;
                               chkptr   = chk_cmd( cmdword );
                               if ( chkptr->dirn == D_CARD )
                               {   /** data to card **/
                                  waitbyte += rxlen;
                                  /** set_state(S_WAITCMD); **/
                                  set_state( S_SENDTERM );
                               }
                               else
                               {   /** data to decoder **/
                                   next_state = S_SENDTERM;
                                   set_state(S_SENDSTR);
                                   txlen      = rxlen;
                                   txptr      = chkptr->dataptr;
                               }
                             }
                             break;
       case    S_SENDTERM  : next_state  = S_WAITCMD;
                             set_state(S_SENDSTR);
                             txlen       = 2;
                             txptr       = terminator;
                             break;
       case    S_WAIT      : if (( rxval = com2rd(fn)) != -1)
                             {
                                 printf("%2.2X ",rxval);
                                 cmdword = rxval << 8;
                             }
                             break;
       }
       if ( com2stat & ST_ERR )
       {
           if ( com2stat & ST_BRK )
               if (showall) printf("brk");
           if ( com2stat & ST_FRM )
               if (showall) printf("frm");
           if ( com2stat & ST_PTY )
               if (showall) printf("pty");
           if ( com2stat & ST_OVR )
               if (showall) printf("ovr");
           com2stat &= ~ST_ERR;
       }
   }

   rsetint1b();
   resetint(COM1INT + COM2INT + TIMEINT );

   if ( outfile != NULL )
       fclose( outfile );

   printf("fn = %d\n",fn);
}


void set_state( byte newstate )
{
   int i;

   state = newstate;
   /** for ( i=0; i<500; i++); **/

   /****************
   switch ( state )
   {
   case S_RESET    : printf("RESET\n");
                     break;
   case S_RELEASED : printf("RELEASED\n");
                     break;
   case S_STIMER   : printf("TIMER\n");
                     break;
   case S_SENDSTR  : printf("SENDSTR\n"); 
                     break;
   case S_SENDECHO : printf("SENDECHO\n");
                     break;
   case S_WAITCMD  : printf("WAITCMD\n");
                     break;
   case S_GETCMD   : printf("GETCMD\n");
                     break;
   case S_GETCMD2  : printf("GETCMD2\n");
                     break;
   case S_GETLEN   : printf("GETLEN\n");
                     break;
   case S_CMDREPLY : printf("CMDREPLY\n");
                     break;
   case S_SENDTERM : printf("SENDTERM\n");
                     break;
   case S_WAIT     : printf("WAIT\n");
                     break;
   }
   **********/
}




/** Check a command and update its history. **/

cmddef * chk_cmd( word command )
{
   cmddef * ptr;
   byte i;

   i     = 0;

   while ( instructs[i].cmdword != 0 )
   {
       if ( instructs[i].cmdword == command )
           return( &instructs[i] );
       else i++;
   }
   instructs[i].cmdword = command;
   instructs[i].dirn    = D_CARD;
   instructs[i+1].cmdword = 0;
   return( &instructs[i] );
}

int startcmdwd( unsigned int first )
{
   byte i;

   first <<= 8;
   i = 0;
   while ( instructs[i].cmdword != 0 )
   {
       if ( (instructs[i].cmdword & 0xff00) == first )
           return( 1 );
       else
           i++;
   }
   return( 0 );
}

