// Nagravision/Syster, oh well, why not... ;-)

#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <graphics.h>
#include <dos.h>
#include "svga256.h"

#define LINE_LEN 512
#define FRAME_LEN 512
#define START_LINE 0
#define LEVELS 4

typedef struct
{
	unsigned char **mean;
	int *lineUsed;
	int *lineOrder;
} CheckSum;

typedef unsigned char * ucharptr;

void InitSVGA();
int huge DetectVGA256();
void buildinitpal(DacPalette256 *svga_pal);
void WriteTable(char *fileName, CheckSum *cs, int slices);
long cmp(CheckSum *cs, int i, int j, int slices, int threshold);

int main(int argc, char **argv)
{
	FILE *inFile;
	unsigned char dummyBuffer[256];
	int grayValue;
	CheckSum *cs = new CheckSum;
	fpos_t begin;

	if(argc!=4)
	{
		printf("\nUsage: nagra <filename.pgm> slices threshold\n\n");
		printf("\tslices    [2-512] - 512 slowest and best result\n");
		printf("\t          must be powers of 2 (eg. 128 256 512)\n");
		printf("\tthreshold [0-256] - 0 best result\n");
		exit(1);
	}

	// open file
	if( (inFile=fopen(argv[1],"rb")) == NULL)
	{
		printf("Error opening file!!!\n");
		exit(1);
	}

	int slices = atoi(argv[2]);
	int threshold = atoi(argv[3]);

	cs->lineUsed = new int[FRAME_LEN];
	cs->lineOrder = new int[FRAME_LEN];
	cs->mean = new ucharptr[slices];

	for(int i=0; i<slices; i++)
	{
		cs->mean[i] = new unsigned char[FRAME_LEN];
	}

	InitSVGA();

	// read header info
	for(i=0; i<3; i++)
	{
		fgets(dummyBuffer,256,inFile);
	}
	fgetpos(inFile, &begin);

	// display encoded screen
	int k;
	for(int j=0; j<FRAME_LEN; j++)
	{
		for(i=0; i<LINE_LEN; i+=LINE_LEN/slices)
		{
			if(slices == LINE_LEN)
			{
				grayValue = fgetc(inFile);
				grayValue = (grayValue / (256/LEVELS) ) * (255/(LEVELS-1));
				putpixel(i,j,grayValue);
				cs->mean[i][j] = grayValue;
			}
			else
			{
				int sum = 0;
				for(int k=0; k<LINE_LEN/slices; k++)
				{
					grayValue = fgetc(inFile);
					grayValue = (grayValue / (256/LEVELS) ) * (255/(LEVELS-1));
					putpixel(i+k,j,grayValue);
					sum += grayValue;
				}
				cs->mean[i/(LINE_LEN/slices)][j] = sum/(LINE_LEN/slices);
			}
		}
		cs->lineUsed[j] = 0;
		cs->lineOrder[j] = 0;
	}


	cs->lineUsed[START_LINE] = 1;
	cs->lineOrder[0] = START_LINE;

	int ff = 0;
	// process cycle
	long diff = 0l;
	long newDiff;

	for(j=1; j<FRAME_LEN; j++)
	{
		diff=0l;
		for(i=0; i<FRAME_LEN; i++)
		{
			if(  (cs->lineUsed[i]==0) )
			{
				newDiff = cmp(cs, (cs->lineOrder[j-1]), i, slices, threshold);
				if(newDiff > diff)
				{
					diff = newDiff;
					cs->lineOrder[j] = i;
				}
				if(ff)
					putpixel(600,0,255);
				else
					putpixel(600,0,0);
				ff = ~ff;
			}
		}
		cs->lineUsed[ cs->lineOrder[j] ] = 1;
	}

//	WriteTable("table.nag", cs, slices);

	cleardevice();

	sound(500);
	delay(1000);
	nosound();

	// display cycle
	fsetpos(inFile, &begin);
	fpos_t fPos;
	for(j=0; j<FRAME_LEN; j++)
	{
		fPos = begin+(fpos_t)(LINE_LEN*(long)cs->lineOrder[j]);
		fsetpos(inFile, &fPos);
		for(i=0; i<LINE_LEN; i++)
		{
//			putpixel(i,j,cs->mean[i][(cs->lineOrder[j])]);
			putpixel(i,j,fgetc(inFile));
		}
	}

	fclose(inFile);

	getch();

	closegraph();

	delete cs->lineUsed;
	delete cs->lineOrder;
	for(i=0; i<slices; i++)
	{
		delete cs->mean[i];
	}
	delete cs;

	return 0;
}

void InitSVGA()
{
	int Gd = DETECT, Gm, Drv;
	DacPalette256 svga_pal;

	installuserdriver("SVGA256",DetectVGA256);
	initgraph(&Gd,&Gm,"");
	cleardevice();
	buildinitpal(&svga_pal);
}

int huge DetectVGA256()
{
	int Vid=3;

//   printf("Which video mode would you like to use? \n");
//   printf("  0) 320x200x256\n");
//   printf("  1) 640x400x256\n");
//   printf("  2) 640x480x256\n");
//   printf("  3) 800x600x256\n");
//   printf("  4) 1024x768x256\n\n>");
//   scanf("%d",&Vid);
	return Vid;
}

void buildinitpal(DacPalette256 *svga_pal)
{
	int i;

	for(i=0;i<=255;i++)
	{
		(*svga_pal)[i][0]=i/4;
		(*svga_pal)[i][1]=i/4;
		(*svga_pal)[i][2]=i/4;
	}
	setvgapalette(svga_pal);
}

long cmp(CheckSum *cs, int i, int j, int slices, int threshold)
{
	long bonus = 0;
	for(int k=0; k<LINE_LEN; k+=LINE_LEN/slices)
	{
//		if (abs((cs->mean[k][i] - cs->mean[k][j]))<2)
		if (abs((cs->mean[k][i] - cs->mean[k][j])) <= threshold)
		{ bonus++;
		}
	}
	return bonus;
}

void WriteTable(char *fileName, CheckSum *cs, int slices)
{
	FILE *outFile;
	int i,j;

	if( (outFile=fopen(fileName,"wb")) == NULL)
	{
		printf("Error opening file!!!\n");
		exit(1);
	}
	for(j=0; j<FRAME_LEN; j++)
	{
		for(i=0; i<slices; i++)
		{
			fprintf(outFile,"%3d ",(int)cs->mean[i][j]);
		}
		fprintf(outFile," - %1d - %3d\n",cs->lineUsed[j],cs->lineOrder[j]);
	}
	fclose(outFile);
}