/*
 * This module contains the critical algorithm and secret key
 * used by British Sky Broadcasting in their Videocrypt pay-TV
 * chip card in 1993 and early 1994. It is expected that most
 * information in this file becomes obsolete when the next card
 * generation (09) is activated.
 */

#include "decrypt.h"

/*
 * the secret key -- for your eyes only :-)
 */
const unsigned char key[56] = {
  0x65, 0xe7, 0x71, 0x1a, 0xb4, 0x88, 0xd7, 0x76,
  0x28, 0xd0, 0x4c, 0x6e, 0x86, 0x8c, 0xc8, 0x43,
  0xa9, 0xec, 0x60, 0x42, 0x05, 0xf2, 0x3d, 0x1c,
  0x6c, 0xbc, 0xaf, 0xc3, 0x2b, 0xb5, 0xdc, 0x90,
  0xf9, 0x05, 0xea, 0x51, 0x46, 0x9d, 0xe2, 0x60,
  0x70, 0x52, 0x67, 0x26, 0x61, 0x49, 0x42, 0x09,
  0x50, 0x99, 0x90, 0xa2, 0x36, 0x0e, 0xfd, 0x39
};


/*
 * This is the core function of the decryption algorithm
 * which is iterated 99 times by decode(). This code assumes that
 * unsigned char is exactly 8-bit long.
 */
void kernel(unsigned char *out, int *oi, const unsigned char in,
            int offset)
{
  unsigned char b, c;

  out[*oi] ^= in;
  b = key[offset + (out[*oi] >> 4)];
  c = key[offset + (out[*oi] & 0x0f) + 16];
  c = ~(c + b);
  c = (c << 1) | (c >> 7);    /* rotate 1 left */
  c += in;
  c = (c << 1) | (c >> 7);    /* rotate 1 left */
  c = (c >> 4) | (c << 4);    /* swap nibbles */
  *oi = (*oi + 1) & 7;
  out[*oi] ^= c;

  return;
}


/*
 * The decoder requests every ~2.5 seconds an answer to a 32-byte
 * packet (msg) from the chip card. The card's 8-byte answer (answ) to
 * this request is calculated by this function.
 */
int decode(const unsigned char *msg, unsigned char *answ)
{
  int i;
  int oi = 0;           /* index in output array answ[] */
  int offset = 0;       /* secret key table selection   */
  int check = 0;        /* flag for incorrect checksum  */
  unsigned char b = 0;

  if (msg[1] > 0x32) offset = 0x08;
  if (msg[1] > 0x3a) offset = 0x18;
  for (i = 0; i < 8; i++) answ[i] = 0;
  for (i = 0; i < 27; i++)
    kernel(answ, &oi, msg[i], offset);
  for (i = 27; i < 31; i++) {
    kernel(answ, &oi, b, offset);
    kernel(answ, &oi, b, offset);
    b = msg[i];
    if (answ[oi] != msg[i]) check |= 1;
    oi = (oi + 1) & 7;
  }
  /* msg[30] is completely ignored */
  for (i = 0; i < 64; i++)
    kernel(answ, &oi, msg[31], offset);

  /* test checksum */
  b = 0;
  for (i = 0; i < 32; i++)
    b += msg[i];
  if (b != 0) check |= 2;

  return check;
}
